// package com.skylit.io;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.text.DecimalFormat;

/**
 *  @author Gary Litvin
 *  @version 1.0, 10/10/02
 *
 *  Written as part of
 *
 *  <i>Java Methods: An Introduction to Object-Oriented Programming</i>
 *  (Skylight Publishing 2001, ISBN 0-9654853-7-4)
 *
 *   and
 *
 *  <i>Java Methods AB: Data Structures</i>
 *  (Skylight Publishing 2003, ISBN 0-9654853-1-5)
 *
 *  EasyDate provides a simple representation of a date with month, day, year
 *  fields.  EasyDate objects are immutable.
 *
 *  <xmp>
 *  Example:
 *  =======
 *
 *  EasyDate today = new EasyDate();
 *  System.out.println("Today is " + today);
 *
 *  EasyDate tomorrow = today.add(1);
 *  EasyDate yesterday = today.add(-1);
 *
 *  int yr = today.getYear();
 *  System.out.println(yr + " is a leap year? This is " +
 *                                   EasyDate.isLeapYear(yr));
 *
 *  EasyDate myBirthday = new EasyDate(bDayMonth, bDayDay, yr); 
 *
 *  if (today.equals(myBirthday))
 *    System.out.println("Today is my birthday");
 *  else if (yesterday.equals(myBirthday))
 *    System.out.println("My birthday was yesterday");
 *  else
 *  {
 *    if (myBirthday.compareTo(today) < 0)
 *      myBirthday = new EasyDate(bDayMonth, bDayDay, yr + 1);
 *    System.out.println(today.daysTo(myBirthday) +
 *                   " days are left until my next birthday");
 *  }
 *  </xmp>
 *
 */

public class EasyDate implements Comparable, Cloneable
{
  private int month;
  private int day;
  private int year;

  /**
   *  Number of milliseconds in 24 hours
   */  
  public static final long MILLIS_DAY = 24L * 3600 * 1000;

  /**
   *  Constructor.  Fills month, day, and year with values for the current date
   */  
  public EasyDate()
  {
    Calendar calendar = new GregorianCalendar();
    month = calendar.get(Calendar.MONTH) + 1;
    day = calendar.get(Calendar.DATE);
    year = calendar.get(Calendar.YEAR);
  }

  /**
   *  Constructor.  Fills month, day, and year with given values
   *  @param <code>month</code> the month setting (1 - 12)
   *  @param <code>day</code> the day setting (1 - 31)
   *  @param <code>year</code> the year setting (1600 - 2100)
   */
  public EasyDate(int month, int day, int year)
  {
    this.month = month;
    this.day = day;
    this.year = year;
  }

  /**
   *  Constructor.  Copies another date into this.
   *  @param <code>other</code> the date to be copied
   */

  public EasyDate(EasyDate other)
  {
    this.month = other.getMonth();
    this.day = other.getDay();
    this.year = other.getYear();
  }

  /**
   *  Checks whether a given year is a leap year
   *  @param <code>year</code> a year to be checked (1600-2100)
   *  @return true if <code>year</code> is a leap year, false otherwise
   */
  public static boolean isLeapYear(int year)
  {
    return (new GregorianCalendar()).isLeapYear(year);
  }

  /**
   *  Returns the month of this EasyDate
   */
  public int getMonth()
  {
    return month;
  }

  /**
   *  Returns the day of this EasyDate
   */
  public int getDay()
  {
    return day;
  }

  /**
   *  Returns the year of this EasyDate
   */
  public int getYear()
  {
    return year;
  }

  /**
   *  Returns a new EasyDate that is <code>numDays</code> later
   *  (or, if numDays is negative, earlier) than this
   */
  public EasyDate add(int numDays)
  {
    Calendar calendar = new GregorianCalendar(year, month - 1, day);    
    calendar.add(Calendar.DATE, numDays);
    return new EasyDate(calendar.get(Calendar.MONTH) + 1,
             calendar.get(Calendar.DATE), calendar.get(Calendar.YEAR));
  }

  /**
   *  Returns the number of days from this date to <code>other</code>
   */
  public int daysTo(EasyDate other)
  {
    Calendar calendar = new GregorianCalendar(other.getYear(),
                                        other.getMonth() - 1, other.getDay());
    long ms = calendar.getTimeInMillis();
    calendar = new GregorianCalendar(year, month - 1, day);
    ms -= calendar.getTimeInMillis();
    return (int)(Math.round((double)ms / MILLIS_DAY));
  }

  /**
   *  Returns the number of days from the date represented by obj to this
   */
  public int compareTo(Object obj)
  {
    return -daysTo((EasyDate)obj);
  }

  /**
   *  Returns true if obj represents the same date as this, false otherwise
   */
  public boolean equals(Object obj)
  {
    EasyDate other = (EasyDate)obj;
    return month == other.getMonth() && day == other.getDay() && year == other.getYear();
  }

  /**
   *  Returns this date as a string in the mm/dd/yyyy format
   */
  public String toString()
  {
    DecimalFormat dd = new DecimalFormat("00");
    return dd.format(month) + "/" + dd.format(day) + "/" + year;
  }
}
